/* 
 * title : xcube.c - cube showing
 * begin : Thu Nov 15 13:49:12 2001
 * make  : gcc xcube.c -o xcube -lm -I/usr/X11R6/include -L/usr/X11R6/lib -lX11
 * 
 */

#include <math.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define  WIN_WIDTH   640
#define  WIN_HEIGHT  480
#define  V_SCALE     60
#define  H_SCALE     60
#define  NP  8
#define  NL  12

struct COORD {
  double x;
  double y;
  double z;
};

struct CONNECT {
  int bgn;
  int end;
};

struct COORD   coord[NP];
struct CONNECT cnct[NL];


/*  functions  */

double torad(double t)
{
  return (t / 180 * M_PI);
}

int mulmat(double *x, double *y, double th)
{
  double xt, yt, si, co;

  si = sin(th);    co = cos(th);
  xt =   *x * co + *y * si;
  yt = - *x * si + *y * co;
  *x = xt;   *y = yt;
}

int trans(double x, double y, double z, int *px, int *py, double th, double ph)
{
  mulmat(&x, &y, th);
  mulmat(&x, &z, ph);
  *px = WIN_WIDTH  / 2 + (int)(y * H_SCALE);
  *py = WIN_HEIGHT / 2 - (int)(z * V_SCALE);
}

int redraw(Display *d, Window w, GC gc, double theta, double phy)
{
  int x1, y1, x2, y2, i;

  /* ɸ */
  trans(0, 0, 0, &x1, &y1, theta, phy);
  trans(3, 0, 0, &x2, &y2, theta, phy);
  XDrawLine(d, w, gc, x1, y1, x2, y2);
  trans(0, 3, 0, &x2, &y2, theta, phy);
  XDrawLine(d, w, gc, x1, y1, x2, y2);
  trans(0, 0, 3, &x2, &y2, theta, phy);
  XDrawLine(d, w, gc, x1, y1, x2, y2);
  /* ΩΤ */
  for (i = 0; i < NL; i++) {
    trans(
      coord[cnct[i].bgn].x,
      coord[cnct[i].bgn].y,
      coord[cnct[i].bgn].z,
      &x1, &y1, theta, phy);
    trans(
      coord[cnct[i].end].x,
      coord[cnct[i].end].y,
      coord[cnct[i].end].z,
      &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
  }
  XFlush(d);
}


main(int argc, char *argv[])
{
  Display *d;
  Window  root, w;
  XEvent  e;
  GC      gc;
  unsigned int black, white;
  int i;
  int x1, y1, x2, y2;

  double  theta, phy;

  /* coordination set of cube top */
  coord[0].x = 0;  coord[0].y = 0;  coord[0].z = 1;
  coord[1].x = 1;  coord[1].y = 0;  coord[1].z = 1;
  coord[2].x = 1;  coord[2].y = 1;  coord[2].z = 1;
  coord[3].x = 0;  coord[3].y = 1;  coord[3].z = 1;
  coord[4].x = 0;  coord[4].y = 0;  coord[4].z = 0;
  coord[5].x = 1;  coord[5].y = 0;  coord[5].z = 0;
  coord[6].x = 1;  coord[6].y = 1;  coord[6].z = 0;
  coord[7].x = 0;  coord[7].y = 1;  coord[7].z = 0;

  /* relation list of cube top */
  cnct[ 0].bgn = 0;  cnct[ 0].end = 1;
  cnct[ 1].bgn = 1;  cnct[ 1].end = 2;
  cnct[ 2].bgn = 2;  cnct[ 2].end = 3;
  cnct[ 3].bgn = 3;  cnct[ 3].end = 0;
  cnct[ 4].bgn = 0;  cnct[ 4].end = 4;
  cnct[ 5].bgn = 1;  cnct[ 5].end = 5;
  cnct[ 6].bgn = 2;  cnct[ 6].end = 6;
  cnct[ 7].bgn = 3;  cnct[ 7].end = 7;
  cnct[ 8].bgn = 4;  cnct[ 8].end = 5;
  cnct[ 9].bgn = 5;  cnct[ 9].end = 6;
  cnct[10].bgn = 6;  cnct[10].end = 7;
  cnct[11].bgn = 7;  cnct[11].end = 4;

  sscanf(argv[1], "%lf", &theta);
  sscanf(argv[2], "%lf", &phy);

  d= XOpenDisplay(NULL);
  root = RootWindow(d, 0);
  black = BlackPixel(d, 0);
  white = WhitePixel(d, 0);
  w = XCreateSimpleWindow(d, root,  25,  25,
         WIN_WIDTH, WIN_HEIGHT, 2, black, white);
  XSelectInput(d, w, ExposureMask);
  gc = XCreateGC(d, w, 0, 0);
  XMapWindow(d, w);

  while (1) {
    XNextEvent(d, &e);
    if (e.type == Expose)
      redraw(d, w, gc, theta, phy);
  }

}
