/* 
 * title : xcubic.c - cube showing with pointer
 * begin : Sat Nov 17 23:40:14 2001
 * make  : gcc xcubic.c -o xcubic -lm -I/usr/X11R6/include -L/usr/X11R6/lib -lX11
 * 
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define  WIN_WIDTH   640
#define  WIN_HEIGHT  480
#define  V_SCALE     60
#define  H_SCALE     60
#define  NP  300
#define  NL  500

/*  structure  */
struct COORD {
  double x;
  double y;
  double z;
};

struct CONNECT {
  int bgn;
  int end;
};

struct COORD   coord[NP];
struct CONNECT cnct[NL];

int axis = False;

/*  functions  */

double torad(double t)
{
  return (t / 180 * M_PI);
}

int mulmat(double *x, double *y, double th)
{
  double xt, yt, si, co;

  si = sin(th);    co = cos(th);
  xt =   *x * co + *y * si;
  yt = - *x * si + *y * co;
  *x = xt;   *y = yt;
}

int trans(double x, double y, double z, int *px, int *py, double th, double ph)
{
  mulmat(&x, &y, th);
  mulmat(&x, &z, ph);
  *px = WIN_WIDTH  / 2 + (int)(y * H_SCALE);
  *py = WIN_HEIGHT / 2 - (int)(z * V_SCALE);
}

int redraw(Display *d, Window w, GC gc, double theta, double phy)
{
  int x1, y1, x2, y2, i;

  XClearWindow(d, w);
  /* draw axis */
  if (axis == True) {
    trans(0, 0, 0, &x1, &y1, theta, phy);
    trans(3, 0, 0, &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
    trans(0, 3, 0, &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
    trans(0, 0, 3, &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
  }
  /* ΩΤ */
  for (i = 0; i < NL; i++) {
    trans(
      coord[cnct[i].bgn].x,
      coord[cnct[i].bgn].y,
      coord[cnct[i].bgn].z,
      &x1, &y1, theta, phy);
    trans(
      coord[cnct[i].end].x,
      coord[cnct[i].end].y,
      coord[cnct[i].end].z,
      &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
  }

  XFlush(d);
}


int main(int argc, char *argv[])
{
  Display *d;
  Window  root, w;
  XEvent  e;
  GC      gc;
  unsigned int black, white;
  int i, j, idx;
  int x1, y1, x2, y2;
  double  theta, phy;
  FILE *fp;
  char line[1024], filnam[1024];
  int flg = 0;

  idx = 1;
  /* ޥɥ饤󤫤ѥ᥿ */
  while (idx < argc) {
    /* إɽ */
    if (strcmp(argv[idx], "-h") == 0) {
      fprintf(stderr, "*** cubic frame viewer\n");
      fprintf(stderr, "Usage : %s [-a] <data file>\n", argv[0]);
      fprintf(stderr, "options as follows:\n");
      fprintf(stderr, "  [-h]\n");
      fprintf(stderr, "  [-a]\n");
      fprintf(stderr, "see document for more informations, thanks.\n");
      exit(1);
    } else
    if (strcmp(argv[idx], "-a") == 0)
    /* ɸե饰 */
      axis = True;
    else
    /* ե̾ */
      strcpy(filnam, argv[idx]);
    idx++;
  }

  if (strcmp(filnam, "") == 0) {
    fprintf(stderr, "error : not specified data file.\n");
    exit(1);
  }
  
  /* ɸӷǡɤ߹ */
  fp = fopen(filnam, "r");
  i = 0;
  while(feof(fp) == 0) {
    fgets(line, 1024, fp);
#ifdef DEBUG
    printf("%s", line);
#endif
    if (strlen(line) <= 1) {
      flg = 1;
      i = 0;
    }
    else {
      if (flg == 0) {
        sscanf(line, "%lg%lg%lg", &coord[i].x, &coord[i].y, &coord[i].z);
#ifdef DEBUG
        printf("Coord : %f, %f, %f\n", coord[i].x, coord[i].y, coord[i].z);
#endif
      }
      else {
        sscanf(line, "%d%d", &cnct[i].bgn, &cnct[i].end);
#ifdef DEBUG
        printf("Link  : %d - %d\n", cnct[i].bgn, cnct[i].end);
#endif
      }
      i++;
    }
  }
  fclose(fp);

  d= XOpenDisplay(NULL);
  root = RootWindow(d, 0);
  w = XCreateSimpleWindow(d, root,  25,  25,
         WIN_WIDTH, WIN_HEIGHT, 2, BlackPixel(d, 0), WhitePixel(d,0));
  /* ޥܥ󡢥ɥϪСݥ󥿰ưΥȤª */
  XSelectInput(d, w, ButtonPressMask | ExposureMask | PointerMotionMask);
  gc = XCreateGC(d, w, 0, 0);
  XMapWindow(d, w);

  while (1) {
    XNextEvent(d, &e);
    if (e.type == ButtonPress) 
      /* ɥǥޥܥ򲡤顢롼פæƽλ */
      break;
    if (e.type == Expose) {
      /* ɥϪФˤľ */
      redraw(d, w, gc, theta, phy);  
    } else
    if (e.type == MotionNotify) {
      /* ݥ󥿤κɸ */
      i = e.xbutton.x;  j = e.xbutton.y;
      /* ȡդѴ롣Ȥϣžʬˤ */
      theta = torad((i - WIN_WIDTH/2) * 720 / WIN_WIDTH);
      phy   = torad((WIN_HEIGHT/2 - j)* 360 / WIN_HEIGHT);
      /* ٤ľ */
      redraw(d, w, gc, theta, phy);  
#ifdef DEBUG
      printf("x : %d,  y : %d\n", i, j);
      printf("th : %f,  ph : %f\n", theta, phy);
#endif
    }
  }
  XCloseDisplay(d);
}
