/* 
 * title : xcubounc.c - cubic field bounce
 * begin : Sat Apr 13 12:27:27 2002
 * make  : gcc xcubounc.c -o xcubounc -lm -I/usr/X11R6/include -L/usr/X11R6/lib -lX11
 * note  : based on xcubic.c
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* ɥ礭 */
#define  WIN_WIDTH   640
#define  WIN_HEIGHT  480
#define  V_SCALE     60
#define  H_SCALE     60
/* ܡȾ */
#define  RADIUS      20
/* ǥ륿 */
#define  DELTA       0.01
/* ӣξ */
#define  REAL_WIDTH  3.0
/* ܡɸθĿ */
#define  NB  30
/* 顼ٻߴֳ */
#define  ALARM_VAL  100000


/*  structure  */
struct BALL {
  double  x;
  double  y;
  double  z;
  double  vx;
  double  vy;
  double  vz;
};

struct BALL  ball[NB];

Display  *d;
Window   w;
GC       gc;

double  dt = DELTA;
double  g = 9.8;
double  e = 0.9;
double  theta, phy;

int axis = True;

/*  functions  */

double torad(double t)
{
  return (t / 180 * M_PI);
}

int mulmat(double *x, double *y, double th)
{
  double xt, yt, si, co;

  si = sin(th);    co = cos(th);
  xt =   *x * co + *y * si;
  yt = - *x * si + *y * co;
  *x = xt;   *y = yt;
}

int trans(double x, double y, double z, int *px, int *py, double th, double ph)
{
  mulmat(&x, &y, th);
  mulmat(&x, &z, ph);
  *px = WIN_WIDTH  / 2 + (int)(y * H_SCALE);
  *py = WIN_HEIGHT / 2 - (int)(z * V_SCALE);
}

int redraw(Display *d, Window w, GC gc, double theta, double phy)
{
  int x, y, x1, x2, y1, y2, i;

  /*  */
  if (axis == True) {
    trans(0, 0, 0, &x1, &y1, theta, phy);
    trans(3, 0, 0, &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
    trans(0, 3, 0, &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
    trans(0, 0, 3, &x2, &y2, theta, phy);
    XDrawLine(d, w, gc, x1, y1, x2, y2);
  }
  /* ܡ */
  for (i = 0; i < NB; i++) {
    trans(
      ball[i].x, ball[i].y, ball[i].z,
      &x, &y, theta, phy);
    XDrawArc(d, w, gc, x-RADIUS/2, y-RADIUS/2, RADIUS, RADIUS, 0, 360*64);
  }
  XFlush(d);
}

/* ɸ׻ */
int new_coord()
{
  int i;
  /* Ťɸ򤺤餷Ƥ*/
  for (i = 1; i < NB; i++)
    ball[i-1] = ball[i];

  /* ʬˤ֡®٤ι³ */
  /* 褺ϣΰ֤򹹿 */
  ball[NB-1].x += ball[NB-1].vx * dt;
  /* ü˹Ԥä顢ȿͽ */
  if (ball[NB-1].x < 0) {
    ball[NB-1].x = 0;
    ball[NB-1].vx = -e * ball[NB-1].vx;
  } else
  /* ü˹Ԥä顢ȿͽ */
  if (ball[NB-1].x >= REAL_WIDTH) {
    ball[NB-1].x = REAL_WIDTH;
    ball[NB-1].vx = -e * ball[NB-1].vx;
  }
  /* 褺ϣΰ֤򹹿 */
  ball[NB-1].y += ball[NB-1].vy * dt;
  /* ü˹Ԥä顢ȿͽ */
  if (ball[NB-1].y < 0) {
    ball[NB-1].y = 0;
    ball[NB-1].vy = -e * ball[NB-1].vy;
  } else
  /* ü˹Ԥä顢ȿͽ */
  if (ball[NB-1].y >= REAL_WIDTH) {
    ball[NB-1].y = REAL_WIDTH;
    ball[NB-1].vy = -e * ball[NB-1].vy;
  }
  /* ³ƣι */
  ball[NB-1].z += ball[NB-1].vz * dt;
  /* ã顢ȿͽ */
  if (ball[NB-1].z < 0) {
    ball[NB-1].z = 0;
    ball[NB-1].vz = -e * ball[NB-1].vz;
  }
  /* ϲ®٤ˤ®٤ι */
  ball[NB-1].vx += 0;
  ball[NB-1].vz += -g * dt;
}

/* ʥ */
void sig_happen()
{
  static int i = 0;

  /* ɸ׻ */
  new_coord();
  redraw(d, w, gc, theta, phy);  
  /* (вᥳ޿ɽ) */
  fprintf(stderr, "%d\n", i++);
  /* 顼ॷʥ */
  if (signal(SIGALRM, sig_happen) == SIG_ERR) {
    perror("signal");
    exit(1);
  }
  /* Υ顼 */
  ualarm(ALARM_VAL, 0);
}

int main(int argc, char *argv[])
{
  XEvent  e;
  int i, j, idx;
  int x1, y1, x2, y2;
  int flg = 0;

  idx = 1;
  /* ޥɥ饤󤫤ѥ᥿ */
  while (idx < argc) {
    /* إɽ */
    if (strcmp(argv[idx], "-h") == 0) {
      fprintf(stderr, "*** cubic bounce\n");
      fprintf(stderr, "Usage : %s [-a] \n", argv[0]);
      fprintf(stderr, "options as follows:\n");
      fprintf(stderr, "  [-h]\n");
      fprintf(stderr, "  [-a]\n");
      fprintf(stderr, "see document for more informations, thanks.\n");
      exit(1);
    } else
    if (strcmp(argv[idx], "-a") == 0)
    /* ɸե饰 */
      axis = True;

    idx++;
  }

  /* ܡ֡®٤ν */
  for (i = 0; i < NB; i++) {
    ball[i].x = 0;
    ball[i].y = 0;
    ball[i].z = 3;
    ball[i].vx = 30;
    ball[i].vy = 20;
    ball[i].vz = 3;
  }
  
  d= XOpenDisplay(NULL);
  w = XCreateSimpleWindow(d, RootWindow(d, 0),  25,  25,
         WIN_WIDTH, WIN_HEIGHT, 2, BlackPixel(d, 0), WhitePixel(d,0));
  XSelectInput(d, w, ButtonPressMask | ExposureMask | PointerMotionMask);
  gc = XCreateGC(d, w, 0, 0);
  XMapWindow(d, w);
  while (1) {
    XNextEvent(d, &e);
    if (e.type == Expose) break;
  }
  /* ޥݥ󥿤򥦥ɥذư */
  XWarpPointer(d, None, w, 0, 0, 0, 0, WIN_WIDTH/2, WIN_HEIGHT/2);

  /* 顼ॷʥ */
  if (signal(SIGALRM, sig_happen) == SIG_ERR) {
    perror("signal");
    exit(1);
  }
  /* 顼 */
  ualarm(ALARM_VAL, 0);

  while (1) {
    XNextEvent(d, &e);
    /* ɥǥޥܥ򲡤顢롼פæƽλ */
    if (e.type == ButtonPress) 
      break;
    /* ɥϪФˤľ */
    if (e.type == Expose) {
      XClearWindow(d, w);
      redraw(d, w, gc, theta, phy);  
    } else
    /* ޥݥ󥿤ư顢Ѵ׻ơľ */
    if (e.type == MotionNotify) {
      XClearWindow(d, w);
      i = e.xbutton.x;  j = e.xbutton.y;
      theta = torad((i - WIN_WIDTH/2) * 720 / WIN_WIDTH);
      phy   = torad((WIN_HEIGHT/2 - j)* 360 / WIN_HEIGHT);
      redraw(d, w, gc, theta, phy);  
    }
  }
  XCloseDisplay(d);
}
